struct Assignment {
    lower_border: usize,
    upper_border: usize,
}

impl Assignment {
    fn contains(&self, other: &Assignment) -> bool {
        self.lower_border <= other.lower_border && self.upper_border >= other.upper_border
    }

    fn overlaps(&self, other: &Assignment) -> bool {
        let overlaps_lower = other.upper_border >= self.lower_border && other.upper_border <= self.lower_border;
        let overlaps_upper = other.lower_border >= self.lower_border && other.lower_border <= self.upper_border;

        overlaps_lower || overlaps_upper
    }

    fn parse(input: &str) -> Assignment {
        match input.trim().split_once("-") {
            Some((low, upp)) => Assignment {
                lower_border: low.parse::<usize>().unwrap(),
                upper_border: upp.parse::<usize>().unwrap(),
            },
            None => panic!(),
        }
    }
}

pub fn task_one(input: &str) -> String {
    let mut count = 0;

    for line in input.lines() {
        if let Some((first, second)) = line.split_once(",") {
            let first = Assignment::parse(first);
            let second = Assignment::parse(second);

            if first.contains(&second) || second.contains(&first) {
                count += 1;
            }
        }
    }

    count.to_string()
}

pub fn task_two(input: &str) -> String {
    let mut count = 0;

    for line in input.lines() {
        if let Some((first, second)) = line.split_once(",") {
            let first = Assignment::parse(first);
            let second = Assignment::parse(second);

            if first.overlaps(&second) || second.overlaps(&first) {
                count += 1;
            }
        }
    }

    count.to_string()
}
