use std::collections::{HashMap, VecDeque};

const AMOUNT_OF_STACKS: usize = 9;
const MAX_STACK_HEIGHT: usize = 8;

fn transform_starting_point(input: &str) -> HashMap<usize, VecDeque<char>> {
    let mut collection_of_stacks: HashMap<usize, VecDeque<char>> = HashMap::new();

    let mut start_situation: Vec<&str> = Vec::new();

    for line in input.lines() {
        if line.is_empty() {
            break;
        }
        start_situation.push(line);
    }

    for crate_nr in 1..=MAX_STACK_HEIGHT {
        let line = start_situation[MAX_STACK_HEIGHT - crate_nr];
        let chars: Vec<char> = line.chars().collect();

        for stack_nr in 1..=AMOUNT_OF_STACKS {
            let c = chars[1 + ((stack_nr - 1) * 4)];
            if crate_nr == 1 {
                collection_of_stacks.insert(stack_nr, VecDeque::new());
            }

            if c.is_ascii_alphabetic() {
                collection_of_stacks
                    .get_mut(&stack_nr)
                    .unwrap()
                    .push_back(c);
            }
        }
    }

    // println!("{:?}", collection_of_stacks.get(&1).unwrap());
    // println!("{:?}", collection_of_stacks.get(&2).unwrap());
    // println!("{:?}", collection_of_stacks.get(&3).unwrap());
    // println!("{:?}", collection_of_stacks.get(&4).unwrap());
    // println!("{:?}", collection_of_stacks.get(&6).unwrap());

    collection_of_stacks
}

pub fn task_one(input: &str) -> String {
    let mut collection_of_stacks = transform_starting_point(&input);

    for line in input.lines().skip(MAX_STACK_HEIGHT + 2) {
        if line.is_empty() {
            continue;
        }

        let mut splitter = line.split_whitespace();
        let amount = splitter.nth(1).unwrap();
        let from = splitter.nth(1).unwrap();
        let to = splitter.nth(1).unwrap();

        for _ in 1..=amount.parse::<usize>().unwrap() {
            let source_stack = collection_of_stacks
                .get_mut(&from.parse::<usize>().unwrap())
                .unwrap();
            let c = source_stack.pop_back().unwrap();

            let target_stack = collection_of_stacks
                .get_mut(&to.parse::<usize>().unwrap())
                .unwrap();
            target_stack.push_back(c);
        }
    }

    let mut solution = String::new();

    for i in 1..=AMOUNT_OF_STACKS {
        solution.push(
            collection_of_stacks
                .get_mut(&i)
                .unwrap()
                .pop_back()
                .unwrap(),
        );
    }

    solution
}

pub fn task_two(input: &str) -> String {
    let mut collection_of_stacks = transform_starting_point(&input);

    for line in input.lines().skip(MAX_STACK_HEIGHT + 2) {
        if line.is_empty() {
            continue;
        }

        let mut splitter = line.split_whitespace();
        let amount = splitter.nth(1).unwrap();
        let from = splitter.nth(1).unwrap();
        let to = splitter.nth(1).unwrap();

        let source_stack = collection_of_stacks
            .get_mut(&from.parse::<usize>().unwrap())
            .unwrap();
        let mut cache: VecDeque<char> = VecDeque::new();

        for _ in 1..=amount.parse::<usize>().unwrap() {
            let c = source_stack.pop_back().unwrap();
            cache.push_front(c);
        }
        let target_stack = collection_of_stacks
            .get_mut(&to.parse::<usize>().unwrap())
            .unwrap();

        while let Some(c) = cache.pop_front() {
            target_stack.push_back(c);
        }
    }

    let mut solution = String::new();

    for i in 1..=AMOUNT_OF_STACKS {
        solution.push(
            collection_of_stacks
                .get_mut(&i)
                .unwrap()
                .pop_back()
                .unwrap(),
        );
    }

    solution
}
